###########################################################
#
# Copyright (c) 2005-2010, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

__all__ = ['CacheStartup']

import tacticenv

from pyasm.common import Config
from pyasm.search import DbContainer
from tactic.command import Scheduler, SchedulerTask
from pyasm.security import Batch
from pyasm.biz import BaseCache, CacheContainer, SearchTypeCache

# custom cache class
class TableInfoCache(BaseCache):
    def __init__(my, **kwargs):
        my.key = kwargs.get("key")
        my.database = kwargs.get("database")
        my.tables = kwargs.get("tables")
        super(TableInfoCache, my).__init__(my.key)

    def init_cache(my):
        from pyasm.search import DatabaseImpl, DbContainer
        my.caches = {}


        data = {}

        for table in my.tables:
            column_data = DatabaseImpl.get().get_column_info(my.database, table, use_cache=False)
            data[table] = column_data
        #my.caches['data'] = data

        # get the order columns
        columns = {}
        sql = DbContainer.get(my.database)
        for table in my.tables:
            column_list = sql.get_columns(table, use_cache=False)
            columns[table] = column_list

        my.caches = {}
        my.caches['data'] = data
        my.caches['columns'] = columns



class CacheStartup(object):
    '''do a bunch of caching on startup of the web process'''

    def execute(my):

        # cache sthpw tables definitions
        data = CacheContainer.get("sthpw_column_info")
        if data == None:
            tables = ['project', 'search_object', 'login', 'login_group', 'login_in_group','snapshot','file','trigger','notification','ticket']
            kwargs = {
                "key": 'sthpw_column_info',
                "database": 'sthpw',
                "tables": tables,
            }
            cache = TableInfoCache( **kwargs )



        # cache search object table
        search_type_cache = SearchTypeCache.get("sthpw/search_object")
        search_type_cache.build_cache_by_column("search_type")

        # cache login table
        search_type_cache = SearchTypeCache.get("sthpw/login")
        search_type_cache.build_cache_by_column("login")

        # cache login_group table
        search_type_cache = SearchTypeCache.get("sthpw/login_group")
        search_type_cache.build_cache_by_column("login_group")

        # DISABLING this ... needs to be written often and is large
        #search_type_cache = SearchTypeCache.get("sthpw/ticket")
        #search_type_cache.build_cache_by_column("ticket")





    def init_scheduler(my):

        scheduler = Scheduler.get()

        # do a dirty check every X seconds
        class DirtyTask(SchedulerTask):
            def execute(my):
                #Batch()

                dirty_caches = CacheContainer.get_dirty()
                for dirty_cache in dirty_caches:
                    key = dirty_cache.get_value("key")
                    #print "... dirtry caching: ", key
                    cache = CacheContainer.get(key)
                    if not cache:
                        print "WARNING: cache [%s] does not exist in memory" % key
                        continue

                    cache.init_cache()

                DbContainer.commit_thread_sql()
                DbContainer.close_all()

        task = DirtyTask()
        interval = 30
        scheduler.add_interval_task(task, interval=interval, mode='threaded', delay=10)

        # do a full cache refresh every 180 seconds
        class RefreshTask(SchedulerTask):
            def execute(my):
                import time
                start = time.time()
                #Batch()

                #print "refresh caches ..."
                caches = CacheContainer.get_all_caches()
                for key, cache in caches.items():
                    #print "... refresh cache %s" % key
                    cache.init_cache()
                #print "... %s seconds" % (time.time() - start)

                DbContainer.commit_thread_sql()
                DbContainer.close_all()

        task = RefreshTask()
        interval = 180
        scheduler.add_interval_task(task, interval=interval, mode='threaded', delay=30)


        # close all extraneous database connections 15 minutes
        class DatabaseCloseTask(SchedulerTask):
            def execute(my):
                #print "Closing all connections"
                DbContainer.close_all_global_connections()

        task = DatabaseCloseTask()
        interval = 15*60
        scheduler.add_interval_task(task, interval=interval, mode='threaded', delay=60)



        # Kill cherrypy every interval.  This overcomes some of the memory
        # problems with long running Python processes.  In order to
        # use this properly, it is essential that a load balancer with
        # proper failover is used
        #
        class KillTacticTask(SchedulerTask):
            def execute(my):
                import cherrypy
                print
                print "Stopping TACTIC ..."
                print 
                scheduler.stop()
                cherrypy.engine.stop()
                cherrypy.engine.exit()
                DbContainer.close_all_global_connections()


        from web_container import WebContainer
        if not WebContainer.is_dev_mode():
            task = KillTacticTask()
            delay = Config.get_value("services", "process_time_alive")
            if delay:
                import random
                
                # put in a randomizer so that not all processes die at once
                delay = int(delay)
                offset = random.randint(0, delay) - delay/2
                delay += offset
                delay = delay * 60
                scheduler.add_single_task(task, mode='sequential', delay=delay)


        print "Starting Scheduler ...."
        scheduler.start_thread()










if __name__ == '__main__':
    Batch(project_code='cg')
    cmd = CacheStartup()
    cmd.execute()
    cmd.init_scheduler()

    try:
        import time
        time.sleep(600)
    except KeyboardInterrupt:
        scheduler = Scheduler.get()
        scheduler.stop()

    cache = CacheContainer.get("sthpw_column_info")
    print cache.get_value_by_key('data', 'login')







