###########################################################
#
# Copyright (c) 2010, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#
__all__ = ['PipelineTaskStatusTrigger', 'PipelineTaskTrigger', 'RelatedTaskUpdateTrigger']

import tacticenv

from pyasm.common import Common, Xml, jsonloads
from pyasm.web import Widget, WebContainer, WidgetException
from pyasm.command import Command, CommandException, Trigger

from pyasm.biz import Pipeline
from pyasm.search import Search, SObject, SearchKey

class PipelineTaskStatusTrigger(Trigger):
    # if the "ingest" task is set to "Approved",
    # then set the "work" task to "Pending"
    SAMPLE_DATA = {
        'src_process':  'ingest',
        'src_status':   'Approved',
        'dst_process':  'work',
        'dst_status':   'Pending',
    }


    def execute(my):
        trigger_sobj = my.get_trigger_sobj()
        data = trigger_sobj.get_value("data")
        data = jsonloads(data)
        #print "data: ", data, type(data)

        data_list = data
        if isinstance(data, dict):
            data_list = [data]
        src_task = my.get_caller()
        for data in data_list:
            # get the src task caller
            dst_task = None
            # it could be the FileCheckin Command
            if not isinstance(src_task, SObject):
                input = my.get_input()
                snapshot = input.get('snapshot')

                if not snapshot:
                    continue
                if isinstance(snapshot, dict):
                    snapshot = SearchKey.get_by_search_key(snapshot.get('__search_key__'))
                src_process = data.get('src_process')
                
                src_task = Search.eval("@SOBJECT(parent.sthpw/task['process','%s'])"%src_process,\
                    sobjects=snapshot, single=True)
                if not src_task:
                    continue

            # make sure the caller process is the same as the source process
            if src_task.get_value("process") != data.get("src_process"):
                continue

            # make sure that the appropriate status was set
            src_status = data.get("src_status")
            if src_status and src_task.get_value("status") != src_status:
                continue

            dst_process = data.get("dst_process")

            # find the task with the appropriate process
            if src_task.get_value("process") == dst_process:
                dst_task = src_task

            # get the output and input tasks
            if not dst_task:
                output_tasks = src_task.get_output_tasks()

                for task in output_tasks:
                    if task.get_value("process") == dst_process:
                        dst_task = task
                        break

            if not dst_task:
                input_tasks = src_task.get_input_tasks()
                for task in input_tasks:
                    if task.get_value("process") == dst_process:
                        dst_task = task
                        break

            if not dst_task:
                continue

            dst_status = data.get("dst_status")

            dst_task.set_value("status", dst_status)
            dst_task.commit()



class PipelineTaskTrigger(Trigger):
    '''This is the trigger that is executed on a change'''

    ARGS_KEYS = {
    }


    def execute(my):
        print "/test/tasks_status_trigger"

        trigger_sobj = my.get_trigger_sobj()
        data = trigger_sobj.get_value("data")
        #data = """[
        #{ "prefix": "rule", "name": "status", "value": "Approved" },
        #{ "prefix": "rule", "name": "pipeline" "value": "model" },
        #{ "prefix": "action", "type": "output", "name": "status", "value": "Pending" }
        #]
        #"""

        data = jsonloads(data)
        print "data: ", data
        from tactic.ui.filter import FilterData
        filter_data = FilterData(data)

        task = my.get_caller()

        # check that the process is correct
        trigger_info = filter_data.get_values_by_index("trigger")
        process = trigger_info.get("process")

        if task.get_value("process") != process:
            return



        parent = None

        rules = filter_data.get_values_by_prefix("rule")

        # go through each rule and determine if this trigger applies
        is_valid = True
        for rule in rules:
            attribute = rule.get('name')
            value = rule.get('value')

            if attribute in ['status']:
                # if condition does not match
                if task.get_value(attribute) != value:
                    is_valid = False

            elif attribute in ['pipeline']:
                attribute = 'pipeline_code'
                if parent == None:
                    parent = task.get_parent()
                    if parent == None:
                        continue

                if parent.get_value(attribute) != value:
                    is_valid = False

            else:
                is_valid = False

        if not is_valid:
            return



        # update the data

        #input = my.get_input()
        #update_data = input.get('update_data')
        #status = update_data.get('status')
        #search_key = input.get('search_key')
        #task = Search.get_by_search_key(search_key)


        # get the next process tasks
        output_tasks = task.get_output_tasks()
        input_tasks = task.get_input_tasks()
        actions = filter_data.get_values_by_prefix("action")
        print "actions: ", actions

        for action in actions:
            type = action.get("type")
            attribute = action.get('name')
            value = action.get('value')

            if type == 'output':
                for output_task in output_tasks:
                    #output_status = output_task.get_value("status")
                    output_task.set_value(attribute, value)
                    output_task.commit()


            elif type == 'input':
                for output_task in output_tasks:
                    print "a : ", attribute, value
                    
                    #output_status = output_task.get_value("status")
                    output_task.set_value(attribute, value)
                    output_task.commit()


            elif type == 'process':
                process = action.get("process")

                for input_task in input_tasks:
                    task_process = input_task.get_value("process")
                    if task_process == process:
                        input_task.set_value(attribute, value)
                        input_task.commit()
                        break

                for output_task in output_tasks:
                    task_process = output_task.get_value("process")
                    if task_process == process:
                        output_task.set_value(attribute, value)
                        output_task.commit()
                        break
                


   

class RelatedTaskUpdateTrigger(Trigger):
    def execute(my):

        input = my.get_input()
        search_key = input.get("search_key")
        update_data = input.get("update_data")

        mode = input.get("mode")
        if mode in ['insert','delete','retire']:
            return

        task = Search.get_by_search_key(search_key)

        process = task.get_value("process")
        context = task.get_value("context")
        parent = task.get_parent()

        # find all of the tasks with the same parent and same context
        search = Search("sthpw/task")
        search.add_parent_filter(parent)
        search.add_filter("process", process)
        search.add_filter("context", context)
        tasks = search.get_sobjects()

        for attr, value in update_data.items():
            # skip assigned as this is the only difference between related tasks
            if attr == 'assigned':
                continue

            for task in tasks:
                task_search_key = task.get_search_key()
                # skip the current one
                if task_search_key == search_key:
                    continue
                task.set_value(attr, value)
                # this should run trigger where applicable
                task.commit()





