###########################################################
#
# Copyright (c) 2010, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#

__all__ = ['ProjectStartWdg']

from pyasm.common import Environment, Common
from pyasm.search import Search
from pyasm.web import DivWdg, HtmlElement, SpanWdg, Table, Widget
from pyasm.widget import IconWdg

import os

from tactic.ui.common import BaseRefreshWdg

class ProjectStartWdg(BaseRefreshWdg):
    '''This is the welcome widget widget will appear on creation of a new
    project
    '''

    def get_args_keys(my):
        return {
        }


    def get_section_wdg(my, title, description, image, behavior):

        section_wdg = DivWdg()
        section_wdg.set_round_corners()
        section_wdg.add_border()
        section_wdg.add_style("width: 225px")
        section_wdg.add_style("height: 310px")
        section_wdg.add_style("overflow: hidden")

        title_wdg = DivWdg()
        section_wdg.add(title_wdg)
        title_wdg.add(title)
        title_wdg.add_style("height: 20px")
        title_wdg.add_style("padding: 3px")
        title_wdg.add_style("margin-top: 3px")
        title_wdg.add_style("font-weight: bold")
        title_wdg.add_gradient("background", "background")

        section_wdg.add_color("background", "background")
        section_wdg.add_behavior( {
        'type': 'hover',
        'add_color_modifier': -5,
        'cb_set_prefix': 'spt.mouse.table_layout_hover',
        } )

        desc_div = DivWdg()
        desc_div.add(description)
        desc_div.add_style("padding: 0px 10px 10px 5px")


        div = DivWdg()
        section_wdg.add(div)
        div.add_style("padding: 3px")
        div.add_style("margin: 5px")
        div.add_style("width: 210px")
        div.add_style("height: 170px")
        div.add(image)
        section_wdg.add(desc_div)
        div.add_style("overflow: hidden")

        section_wdg.add_behavior( behavior )
        section_wdg.add_class("hand")

        return section_wdg


    def get_display(my):

        top = DivWdg()
        top.add_border()
        top.add_style("padding: 10px")
        top.add_color("color", "color")
        top.add_color("background", "background")
        top.add_style("height: 550px")


        title = DivWdg()
        title.add("Project Setup")
        title.add_style("font-size: 15px")
        title.add_style("font-weight: bold")
        top.add(title)
        top.add("<hr/><br/>")

        desc = DivWdg()
        top.add(desc)
        #desc.add("To set up a project, you will first have to create a schema and set up the pipeline for this project.  Afterwards, you will need to manage the Side Bar to provide specific views to your users.<br/><br/>")

        desc.add("To set up a TACTIC project, follow the below steps. Begin by creating the schema, then create pipeline(s) for each sType (searchable type) that requires one. The Side Bar can then be set up to provide specific views to various types of users of the project.<br/><br/>")

        desc.add_style("width: 600px")


        table = Table()
        table.add_color("color", "color")
        top.add(table)
        table.add_row()


        td = table.add_cell()
        td.add_style("vertical-align: top")
        td.add_style("padding: 3px")
        title = "Create Schema"
        #description = '''The schema is used to layout the basic components of your project.  Each component represents a list of items that you use in your business everyday.'''

        description = '''The schema is a collection of nodes that layout the basic components of a project. Each node represents a separate list of items (sType) used in this project: ie Assets, Shots, Artwork, etc.'''

        # Each node will contain a list of "items" and will be stored as a table in the database.'''

        #image = "<img src='/context/images/schema_tool_icon.png'/>"
        image = "<img src='/context/images/getting_started_schema.png'/>"
        behavior = {
        'type': 'click_up',
        'cbjs_action': '''
        spt.tab.set_main_body_tab();
        var class_name = 'tactic.ui.tools.SchemaToolWdg';
        var kwargs = {};
        spt.tab.add_new("create_schema", "Create Schema", class_name, kwargs);
        '''
        }
        schema_wdg = my.get_section_wdg(title, description, image, behavior)
        td.add(schema_wdg)


        td = table.add_cell()
        td.add_style("vertical-align: top")
        td.add_style("padding: 3px")
        title = "Create Workflow"
        #image = "<img src='/context/images/pipeline_tool_icon.png'/>"
        image = "<img src='/context/images/getting_started_pipeline.png'/>"
        #description = "Pipelines define how a particular component will move through its life cycle. Creating pipelines will also allow you to set up statuses, triggers, and notifications for each process."

        description = "Pipelines define how particular items of an sType will move through its lifecycle. Creating pipelines will also allow you to set up automatic triggers and notifications for each process."

        behavior = {
        'type': 'click_up',
        'cbjs_action': '''
        spt.tab.set_main_body_tab();
        var class_name = 'tactic.ui.tools.PipelineToolWdg';
        var kwargs = {};
        spt.tab.add_new("create_workflow", "Create Workflow", class_name, kwargs);
        '''
        }
        pipeline_wdg = my.get_section_wdg(title, description, image, behavior)
        td.add(pipeline_wdg)



        td = table.add_cell()
        td.add_style("padding: 3px")
        td.add_style("vertical-align: top")
        title = "Manage Side Bar"
        #image = "<img src='/context/images/side_bar_icon.png'/>"
        image = "<img src='/context/images/getting_started_sidebar.png'/>"
        description = "The Side Bar can be easily configured to show specific views of your project to each user."
        behavior = {
        'type': 'click_up',
        'cbjs_action': '''
        spt.tab.set_main_body_tab();
        var class_name = 'tactic.ui.panel.ManageViewPanelWdg';
        var kwargs = {};
        spt.tab.add_new("manage_project_views", "Manage Side Bar", class_name, kwargs);
        '''
        }

        side_bar_wdg = my.get_section_wdg(title, description, image, behavior)
        td.add(side_bar_wdg)


        tr, td = table.add_row_cell()
        td.add_style("font-size: 14px")
        div = DivWdg()
        div.add("<br/>"*2)
        title = DivWdg()
        div.add(title)
        title.add("Quick Links")
        title.add_style("font-size: 16px")
        div.add("<hr/>")

        hover = title.get_color("background", -10)


        link_div = DivWdg()
        div.add(link_div)
        link = HtmlElement.href("TACTIC Documentation", "/doc/", target="_blank")
        link.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            spt.help.set_top();
            spt.help.load_alias("main");
            '''
        } )

        link.add_behavior( {
        'type': 'hover',
        'color': hover,
        'cbjs_action_over': '''
        bvr.src_el.setStyle("background", bvr.color);
        ''',
        'cbjs_action_out': '''
        bvr.src_el.setStyle("background", "");
        '''
        } )
        link_div.add(link)
        link.add_color("color", "color")
        link_div.add("<br/>")

        link = HtmlElement.href("Southpaw Web Site", "http://www.southpawtech.com", target="_blank")
        link_div.add(link)
        link.add_color("color", "color")
        link.add_behavior( {
        'type': 'hover',
        'color': hover,
        'cbjs_action_over': '''
        bvr.src_el.setStyle("background", bvr.color);
        ''',
        'cbjs_action_out': '''
        bvr.src_el.setStyle("background", "");
        '''
        } )


        link_div.add("<br/>")


        link = HtmlElement.href("TACTIC Support Community", "http://support.southpawtech.com", target="_blank")
        link_div.add(link)
        link.add_color("color", "color")
        link.add_behavior( {
        'type': 'hover',
        'color': hover,
        'cbjs_action_over': '''
        bvr.src_el.setStyle("background", bvr.color);
        ''',
        'cbjs_action_out': '''
        bvr.src_el.setStyle("background", "");
        '''
        } )


        td.add(div)

        return top



    def get_totals_wdg(my):
        div = DivWdg()
        

        div.add("Number of Pipelines: 3<br/>") 
        div.add("Number of Search Types: 6<br/>") 
        div.add("Number of Naming Conventions: 4<br/>") 
        div.add("Number of Triggers: 8<br/>") 


        return div




