###########################################################
#
# Copyright (c) 2005-2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

# 

__all__ = ['SimpleSearchExampleWdg', 'SimpleSearchWdg']

from pyasm.common import Common, Container, TacticException
from pyasm.search import Search, SearchKey, SearchType
from pyasm.biz import ExpressionParser
from pyasm.web import DivWdg, HtmlElement, Table
from pyasm.widget import HiddenWdg

from tactic.ui.filter import FilterData, BaseFilterElementWdg
from tactic.ui.common import BaseTableElementWdg, BaseRefreshWdg
from tactic.ui.widget import TextBtnSetWdg, ActionButtonWdg

from tactic.ui.filter import KeywordFilterElementWdg


class SimpleSearchExampleWdg(BaseRefreshWdg):

    def get_args_keys(my):
        return {
        'search_type': 'search type for this search widget',
        'search_view': 'view for the search',
        'prefix': 'the prefix used to find the input data'
        }


    def get_display(my):
        top = DivWdg()
        my.set_as_panel(top)

        search_class = 'tactic.ui.app.simple_search_wdg.SimpleSearchWdg'

        # for now add a table layout wdg
        from tactic.ui.panel import ViewPanelWdg
        my.view = 'manage'
        my.search_type = my.kwargs.get("search_type")
        table = ViewPanelWdg(search_type=my.search_type, view=my.view, search_class=search_class)
        top.add(table)

        return top



class SimpleSearchWdg(BaseRefreshWdg):

    def get_args_keys(my):
        return {
        'search_type': 'search type for this search widget',
        'run_search_bvr': 'run search bvr when clicking on Search',
        'prefix': 'the prefix used to find the input data'
        }


    def init(my):
        my.prefix = my.kwargs.get('prefix')
        if not my.prefix:
            my.prefix = 'simple_search'

        my.content = None
        my.top = DivWdg()

        my.prefix = 'simple'
        my.search_type = my.kwargs.get("search_type")
        

    def handle_search(my):
        my.search = Search(my.search_type)
        my.alter_search(my.search)
        return my.search

    def get_search(my):
        return my.search

    def alter_search(my, search):
        '''
        from tactic.ui.filter import BaseFilterElementWdg
        config = my.get_config()
        element_names = config.get_element_names()
        for element_name in element_names:
            widget = config.get_display_widget(element_name)

            if isinstance(widget, BaseFilterElementWdg):
                widget.alter_search(search)

        '''


        from tactic.ui.panel import CustomLayoutWdg


        # define a standard search action
        from tactic.ui.filter import FilterData
        filter_data = FilterData.get()
        config = my.get_config()

        data_list = filter_data.get_values_by_prefix(my.prefix)
        #columns = search.get_columns()

        element_data_dict = {}
        for data in data_list:
            handler = data.get("handler")
            element_name = data.get("element_name")
            if not element_name:
                continue

            element_data_dict[element_name] = data



        element_names = config.get_element_names()
        for element_name in element_names:

            widget = config.get_display_widget(element_name)
            if not widget:
                widget = KeywordFilterElementWdg()
                widget.set_name(element_name)

            data = element_data_dict.get(element_name)
            if not data:
                data = {}
            widget.set_values(data)

            widget.alter_search(search)


        return



        # element_name is supposed to be unique, so build a
        # dictionary of element_name: data
        for data in data_list:
            handler = data.get("handler")
            element_name = data.get("element_name")
            if not element_name:
                continue

            widget = config.get_display_widget(element_name)
            if not widget:
                widget = KeywordFilterElementWdg()
                widget.set_name(element_name)

            widget.set_values(data)

            if isinstance(widget, BaseFilterElementWdg):
                widget.alter_search(search)
                continue
            # FIXME: special provision for a custom layout
            if isinstance(widget, CustomLayoutWdg):
                widget.alter_search(search)
                continue


            # TODO: this would be better done as a check by the widget
            # itself
            """
            #TODO: some comments here please to explain why such
            # condition is not needed in get_display()
            element_name = data.get("element_name")
            if element_name not in columns:
                print "WARNING: skipping [%s] ... not in [%s]" % (element_name, search.get_search_type() )
                continue
            """

            value = data.get(element_name)
            if value == None:
                value = data.get("value")

            op = data.get('op')
            
            if value:
                search.add_filter(element_name, value, op=op)



    def set_content(my, content):
        my.content = content

    def get_top(my):
        top = my.top
        top.add_color("background", "background")
        border_color = top.get_color("border")
        top.add_style("border-width: 1px 0px 1px 0px")
        top.add_style("border-style: solid")
        top.add_style("border-color: %s" % border_color)

        table = Table()
        top.add(table)
        table.add_row()

        if not my.content:
            my.content = DivWdg()
            my.content.add("No Content")

        td = table.add_cell()
        td.add(my.content)

        my.content.add_style("min-width: 400px")


        search_wdg = my.get_search_wdg()
        #search_wdg.add_style("float: left")
        td = table.add_cell()
        td.add(search_wdg)
        td.add_color("background", "background", -5)
        td.add_style("padding-left: 15px")
        td.add_style("padding-right: 15px")
        td.add_style("border-style: solid")
        td.add_style("border-color: %s" % td.get_color("border") )
        td.add_style("border-width: 0px 1px 0px 0px" )


        hidden = HiddenWdg("prefix", my.prefix)
        top.add(hidden)
        # this cannot be spt_search as it will confuse spt.dg_table.search_cbk() 
        top.add_class("spt_simple_search")

        return top


    def get_config(my):
        # TEST
        config_xml = '''
        <config>
        <custom_filter>
          <element name='asset_library'>
            <display class='SelectWdg'>
                <query>prod/asset_library|code|code</query>
                <empty>true</empty>
            </display>
          </element>
          <element name='pipeline_code'>
            <display class='SelectWdg'>
                <query>sthpw/pipeline|code|code</query>
                <empty>true</empty>
            </display>
          </element>
        </custom_filter>
        </config>
        '''

        my.view = my.kwargs.get("search_view")
        if not my.view:
            my.view = 'custom_filter'
        #view = "custom_filter"
        search = Search("config/widget_config")
        search.add_filter("view", my.view)
        search.add_filter("search_type", my.search_type)
        config_sobj = search.get_sobject()
        if config_sobj:
            config_xml = config_sobj.get_value("config")
        else:
            config_xml = '''
            <config>
            <custom_filter>
            </custom_filter>
            </config>
            '''


        from pyasm.widget import WidgetConfig
        config = WidgetConfig.get(view=my.view, xml=config_xml)

        return config

    def get_display(my):

        element_data_dict = {}

        config = my.get_config()
        element_names = config.get_element_names()
        content_wdg = DivWdg()
        content_wdg.add_style("width: 100%")

        if not element_names:
            content_wdg.add_style("height: 18px")
            content_wdg.add("<i>-- No elements defined for simple search --</i>")
            content_wdg.add_style("padding: 10px")
            return content_wdg

        my.set_content(content_wdg)
       
        
        # this is somewhat duplicated logic from alter_search, but since this is called 
        # in ViewPanelWdg, it's a diff instance and needs to retrieve again
        filter_data = FilterData.get()
        data_list = filter_data.get_values_by_prefix(my.prefix)
        for data in data_list:
            handler = data.get("handler")
            element_name = data.get("element_name")
            if not element_name:
                continue

            element_data_dict[element_name] = data

        elements_wdg = DivWdg()
        content_wdg.add(elements_wdg)
        elements_wdg.add_color("color", "color")
        elements_wdg.add_color("background", "background", -5)
        elements_wdg.add_style("width: 100%")
        #elements_wdg.set_round_corners(corners=['TL','TR'])
        elements_wdg.add_style("border-style: solid")
        elements_wdg.add_style("border-color: %s" % elements_wdg.get_color("border") )
        elements_wdg.add_style("border-width: 0px 1px 0px 0px" )
        elements_wdg.add_style("margin-top: -1px" )

        table = Table()
        table.add_color("color", "color")
        elements_wdg.add(table)
       
        num_rows = int(len(element_names)/2)+1
        for i, element_name in enumerate(element_names):

            if i % 2 == 0:
                table.add_row()

            element_wdg = DivWdg()
            #elements_wdg.add(element_wdg)
            table.add_cell(element_wdg)


            element_wdg.add_class("spt_filter_top")


            #if i % 2 == 0:
            if i == 0 and len(element_names) > 1:
                spacer = DivWdg()
                spacer.add_style("border-style: solid")
                spacer.add_style("border-width: 0 1 0 0")
                spacer.add_style("height: %spx" % (num_rows*20))
                spacer.add_style("width: 10px")
                spacer.add_style("border-color: %s" % spacer.get_color("border") )
                spacer.add("&nbsp;")
                td = table.add_cell(spacer)
                td.add_attr("rowspan", num_rows)

            #element_wdg.add_color("color", "color3")
            #element_wdg.add_color("background", "background3")
            #element_wdg.set_round_corners()
            #element_wdg.add_border()

            element_wdg.add_style("padding: 4px 10px 4px 5px")
            element_wdg.add_class("spt_table_search")
            element_wdg.add_style("float: left")
            element_wdg.add_style("margin: 1px")
            element_wdg.add_style("min-height: 15px")

            # this is done at get_top()
            #element_wdg.add_class("spt_search")
            element_wdg.add( HiddenWdg("prefix", my.prefix))

            display_handler = config.get_display_handler(element_name)
            element_wdg.add( HiddenWdg("handler", display_handler))


            element_wdg.add( HiddenWdg("element_name", element_name))
            title = Common.get_display_title(element_name)

            from pyasm.widget import ExceptionWdg
            try:
                widget = config.get_display_widget(element_name)
            except Exception, e:
                element_wdg.add(ExceptionWdg(e))
                continue


            if not widget:
                widget = KeywordFilterElementWdg()
                widget.set_name(element_name)

            from pyasm.widget import IconWdg
            icon_div = DivWdg()
            element_wdg.add(icon_div)
            icon_div.add_style("float: left")
            icon_div.add_style("width: 10px")
            icon_div.add_style("margin-top: 2px")
            #icon_div.add_style("margin-right: -4px")


            widget.set_show_title(True)
            #element_wdg.add("%s: " % title)
            data = element_data_dict.get(element_name)
            if data:
                widget.set_values(data)

            try:
                element_wdg.add(widget.get_buffer_display())
            except Exception, e:
                element_wdg.add(ExceptionWdg(e))
                continue
                


            icon = IconWdg("Filter Set", IconWdg.GREEN_LIGHT)
            icon_div.add(icon)
            icon.add_class("spt_filter_set")

            if not widget.is_set():
                icon.add_style("display: none")



        #elements_wdg.add("<br clear='all'/>")
        top = my.get_top()
        return top



    def get_search_wdg(my):
        search_div = DivWdg()

        if my.kwargs.get('run_search_bvr'):
            run_search_bvr = my.kwargs.get('run_search_bvr')
        else:
            run_search_bvr = {
                'type':         'click_up',
                'cbfn_action':  'spt.dg_table.search_cbk',
                'panel_id':     my.prefix
            }

        button = ActionButtonWdg(title='Search', tip='Run search with this criteria' )
        search_div.add(button)
        button.add_style("margin-top: -7px")
        button.add_behavior( run_search_bvr )

        return search_div





