###########################################################
#
# Copyright (c) 2005-2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#


__all__ = ['TextInputWdg']

from pyasm.common import Date, Common
from pyasm.web import Table, DivWdg, SpanWdg, WebContainer, Widget
from pyasm.widget import IconWdg, TextWdg, BaseInputWdg
from tactic.ui.common import BaseRefreshWdg


import random


class TextInputWdg(BaseInputWdg):

    ARGS_KEYS = {
    'width': {
        'description': 'width of the text element',
        'type': 'TextWdg',
        'order': 1,
        'category': 'Options'
    },
    }
 




    def __init__(my, **kwargs):
        super(TextInputWdg, my).__init__()
        my.kwargs = kwargs

        name = kwargs.get("name")
        my.text = TextWdg(name)

        my.width = my.kwargs.get("width")
        if not my.width:
            my.width = 200
        else:
            my.width = my.width.replace("px", "")
            my.width = int(my.width)

        my.readonly = False


    def add_style(my, name, value=None):
        if not value:
            name, value = name.split(": ")

        if name == 'width':
            my.width = int(value.replace("px",""))
        elif name == 'float':
            my.top.add_style(name, value)
        else:
            my.text.add_style(name, value)


    def set_readonly(my, flag=True):
        my.readonly = flag
        my.text.set_attr("readonly", "readonly")

    def add_class(my, class_name):
        my.text.add_class(class_name)

    def add_color(my, name, palette_key, modifier=0, default=None):
        my.text.add_color(name, palette_key, modifier, default)

    def set_attr(my, name, value):
        my.text.set_attr(name, value)

    def set_value(my, value):
        my.text.set_value(value)

    def get_display(my):

        top = SpanWdg()
        top.add_style("position: relative")
        top.add_class("spt_text_top")
        top.add_style("height: 20px")
        top.add_style("width: %spx" % my.width)

        text_div = SpanWdg()
        top.add(text_div)
        text_div.add(my.text)

        my.text.add_style("-moz-border-radius: 5px")
        my.text.add_style("padding: 2px")
        my.text.add_style("width: %spx" % my.width)
        text_div.add_style("width: %spx" % my.width)

        icon_wdg = DivWdg()
        top.add(icon_wdg)
        icon_wdg.add_style("position: absolute")
        icon_wdg.add_style("left: %spx" % (my.width-18))
        icon_wdg.add_style("top: 1px")


        if not my.readonly:
            icon = IconWdg("Clear", IconWdg.CLOSE_INACTIVE)
            icon.add_class("spt_icon_inactive")
            icon_wdg.add(icon)
            icon = IconWdg("Clear", IconWdg.CLOSE_ACTIVE)
            icon.add_class("spt_icon_active")
            icon.add_style("display: none")
            icon_wdg.add(icon)

            icon_wdg.add_behavior( {
            'type': 'hover',
            'cbjs_action_over': '''
            var inactive = bvr.src_el.getElement(".spt_icon_inactive");
            var active = bvr.src_el.getElement(".spt_icon_active");
            spt.show(active);
            spt.hide(inactive);
            ''',
            'cbjs_action_out': '''
            var inactive = bvr.src_el.getElement(".spt_icon_inactive");
            var active = bvr.src_el.getElement(".spt_icon_active");
            spt.show(inactive);
            spt.hide(active);
            '''
            })



            icon.add_behavior( {
                'type': 'click_up',
                'cbjs_action': '''
                var top = bvr.src_el.getParent(".spt_text_top");
                var text = top.getElement(".spt_input");
                text.value = '';
                '''
            } )

        top.add("&nbsp")
        return top



