###########################################################
#
# Copyright (c) 2005, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#
__all__ = ["HashPanelWdg"]

import os, types, re
from pyasm.common import Xml, XmlException, Common, TacticException, Environment
from pyasm.search import Search

from tactic.ui.common import BaseRefreshWdg


class HashPanelWdg(BaseRefreshWdg):

    def get_args_keys(cls):
        return {
        'hash': 'hash used to build a widget'
        }
    get_args_keys = classmethod(get_args_keys)



    def get_display(my):

        hash = my.kwargs.get("hash")

        options = my.get_options_from_hash(hash)

        class_name = options.get('class_name')
        if not class_name:
            class_name = 'tactic.ui.panel.ViewPanelWdg'
        else:
            del(options['class_name'])

        widget = Common.create_from_class_path(class_name, kwargs=options)
        return widget



    def get_options_from_hash(cls, hash):
        options = {}

        import re
        p = re.compile("^/(\w+)")
        m = p.search(hash)
        if not m:
            return options
        key = m.groups()[0]

        # look up the expression
        search = Search("config/url")
        search.add_filter("url", "/%s/%%"%key, "like")
        search.add_filter("url", "/%s"%key)
        search.add_where("or")
        sobject = search.get_sobject()
        if sobject:
            url = sobject.get_value("url")
            url = url.strip()
            options = Common.extract_dict(hash, url)

            # or try xml
            config = sobject.get_value("widget")

            xml = Xml()
            xml.read_string(config)
            node = xml.get_node("element/display")
            options.update(xml.get_node_values_of_children(node))

            class_name = xml.get_value("element/display/@class")
            if class_name:
                options['class_name'] = class_name

    
        
        # add some predefined ones
        elif key == "link":
            expression = "/link/{link}"
            options = Common.extract_dict(hash, expression)
        elif key == "view":
            expression = "/view/{search_type}/{view}"
            options = Common.extract_dict(hash, expression)
        elif key == "sobject":
            expression = "/sobject/{search_type}/{code}"
            options = Common.extract_dict(hash, expression)
            options["class_name"] = "tactic.ui.panel.SObjectPanelWdg"
        # Only for testn: not very usefull ... not args ...
        # use encoded instead
        elif key == "widget":
            expression = "/widget/{top_class_name}"
            options = Common.extract_dict(hash, expression)


        elif key == "encoded":
            expression = "/encoded/{encoded}"
            options = Common.extract_dict(hash, expression)
            encoded = options['encoded']
            import json, binascii
            data = json.loads( binascii.unhexlify(encoded) )
            class_name = data[0]
            args = data[1]
            kwargs = data[2]

            widget = Common.create_from_class_path(class_name, args, kwargs) 
            options['_widget'] = widget
            

        return options


    get_options_from_hash = classmethod(get_options_from_hash)





